/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2013 Frederik Heber. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * FourBodyPotential_Improper.cpp
 *
 *  Created on: Jul 10, 2013
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Assert.hpp"

#include "FourBodyPotential_Improper.hpp"

#include <boost/assign/list_of.hpp> // for 'map_list_of()'

#include "Potentials/InternalCoordinates/FourBody_ImproperAngle.hpp"

// static definitions
const FourBodyPotential_Improper::ParameterNames_t
FourBodyPotential_Improper::ParameterNames =
      boost::assign::list_of<std::string>
      ("spring_constant")
      ("equilibrium_distance")
    ;
const std::string FourBodyPotential_Improper::improper_token("improper");
Coordinator::ptr FourBodyPotential_Improper::coordinator(Memory::ignore(new FourBody_ImproperAngle()));

static BindingModel generateBindingModel(const EmpiricalPotential::ParticleTypes_t &_ParticleTypes)
{
  // fill nodes
  BindingModel::vector_nodes_t nodes;
  {
    ASSERT( _ParticleTypes.size() == (size_t)4,
        "generateBindingModel() - FourBodyPotential_Improper needs four types.");
    nodes.push_back( FragmentNode(_ParticleTypes[0], 3) );
    nodes.push_back( FragmentNode(_ParticleTypes[1], 1) );
    nodes.push_back( FragmentNode(_ParticleTypes[2], 1) );
    nodes.push_back( FragmentNode(_ParticleTypes[3], 1) );
  }

  // there are no edges
  HomologyGraph::edges_t edges;
  {
    std::pair<HomologyGraph::edges_t::iterator, bool > inserter;
    inserter = edges.insert( std::make_pair( FragmentEdge(_ParticleTypes[0], _ParticleTypes[1]), 1) );
    if (!inserter.second)
      ++(inserter.first->second);
    inserter = edges.insert( std::make_pair( FragmentEdge(_ParticleTypes[0], _ParticleTypes[2]), 1) );
    if (!inserter.second)
      ++(inserter.first->second);
    inserter = edges.insert( std::make_pair( FragmentEdge(_ParticleTypes[0], _ParticleTypes[3]), 1) );
    if (!inserter.second)
      ++(inserter.first->second);
  }

  return BindingModel(nodes, edges);
}

FourBodyPotential_Improper::FourBodyPotential_Improper() :
    FourBodyPotential_Torsion(),
    bindingmodel(BindingModel())
{}

FourBodyPotential_Improper::FourBodyPotential_Improper(
    const ParticleTypes_t &_ParticleTypes) :
    FourBodyPotential_Torsion(_ParticleTypes),
    bindingmodel(generateBindingModel(_ParticleTypes))
{}

FourBodyPotential_Improper::FourBodyPotential_Improper(
    const ParticleTypes_t &_ParticleTypes,
    const double _spring_constant,
    const double _equilibrium_distance) :
    FourBodyPotential_Torsion(
        _ParticleTypes,
        _spring_constant,
        _equilibrium_distance),
    bindingmodel(generateBindingModel(_ParticleTypes))
{}
